// MQOQTOFL.CPP program

//*********************************************************************************************
//*                                                                                           *
//*                            MQOQTOFL.CPP program                                           *
//*                            ====================                                           *
//*                                                                                           *
//*   This program append a Queue to a file, stopping on the First zero-Length record.        *
//*                                                                                           *
//*   The Zero-Length record will act as an End of Queue Indication.                          *
//*                                                                                           *
//*                                                                                           *
//*   The parameters are  /Maaaaaa    -> the name of the Queue Manager                        *
//*                                                  default -> blanks (default QM)           *
//*                                                                                           *
//*                       /Qaaaaaa    -> the name of the Queue                                *
//*                                                  default -> blanks                        *
//*                                                                                           *
//*                       /Faaaaaa    -> the name of the file to append the Queue into        *
//*                                                  default -> blanks                        *
//*                                                                                           *
//*                       /Snnnnnn    -> the Syncpoint Interval                               *
//*                                       /Sb     -> Do a Browse access of the Queue          *
//*                                       /S-     -> Outside of Unit of Work                  *
//*                                       /S0     -> Inside UOW, but Syncpoint at end         *
//*                                       /Snnnn  ->           , Syncpoint after every n mess *
//*                                                                                           *
//*                       /V+         -> Write Diagnotic Messages (default)                   *
//*                       /V-         -> Don't display any messages                           *
//*                                                                                           *
//*                       /Z+         -> Stop on a Zero Length record (default)               *
//*                       /Z-         -> Keep Going when a Zero Length record is reached      *
//*                                                                                           *
//*                                                                                           *
//*   Example : MQOQTOFL /MRAHQM1 /QRAH1 /Fdest.1 /Sb        will create the file 'dest.1'    *
//*                                                          from the queue RAH1 owned by     *
//*                                                          Queue Manager RAHQM1. The        *
//*                                                          Queue is non-destructively       *
//*                                                          processed.                       *
//*                                                                                           *
//*   Standard Messages are written to show information if /V+ is specified.                  *
//*                                                                                           *
//*                                                                                           *
//*                                                                                           *
//*   Return Codes :    0    -> Operation succeeded                                           *
//*                     >0   -> Reason Code for a failure                                     *
//*                                                                                           *
//*                                                                                           *
//*   Copyright : This program is Copyrighted by IBM UK Labs Limited, 1995.                   *
//*                                                                                           *
//*                                                                                           *
//*   Support   : This program is not supported in any way by IBM. It is distributed          *
//*               only to show techniques for Message Queueing. It may be freely              *
//*               modified and adapted by your installation.                                  *
//*                                                                                           *
//*   Author    : Robert Harris,                                                              *
//*               Message Queueing/Performance,                                               *
//*               IBM Hursley Park,                                                           *
//*               England.                                                                    *
//*                                                                                           *
//*********************************************************************************************

//
// System Includes
//

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <time.h>

//
// MQ Includes
//

#include <cmqc.h>

//
// OS2 Includes
//

#define  INCL_DOSMEMMGR
#include <os2.h>

//
// Definitions of datatypes
//

#define FULLWORD     long
#define HALFWORD     int
#define MAXSFULLWORD (68719476735)
#define MAXUFULLWORD (4294967295)
#define MAXSHALFWORD (4095)
#define MAXUHALFWORD (65535)
#define MAXFILENAME  (255)

//
// The parms class contains the unravelling of the
//     supplied parameters
//

class parms
{
   private:

       FULLWORD    verbose                         ; // V flag 0=- 1=+
       FULLWORD    syncint                         ; // S flag -2=browse -1=outside 0=end n=interval
       FULLWORD    zrl                             ; // Z flag 0=- 1=+
       char        qm ??(MQ_Q_MGR_NAME_LENGTH+1??) ; // QM    name
       char        q  ??(MQ_Q_NAME_LENGTH+1    ??) ; // Q     name
       char        fn ??(MAXFILENAME+1         ??) ; // File  name

       FULLWORD parms::getnumb( char * const ptr ) ;    //Private func to get nums out of parms

   public:
                 parms       () { verbose       =  1       ; //Constructor
                                  syncint       =  -2      ; // defaults
                                  zrl           =  1       ; //
                                  memset(qm,0,sizeof(qm))  ; //
                                  memset(q ,0,sizeof(q ))  ; //
                                  memset(fn,0,sizeof(fn))  ; //
                                }

       FULLWORD  isverbose   () { if ( verbose == 1 ) return(TRUE)  ;   //Access func to the pa
                                       else           return(FALSE) ; } ;

       FULLWORD  getsyncint  () { return (syncint) ; } ;

       FULLWORD  getzrl      () { return (zrl    ) ; } ;

       char    * getqmname   (char * const dest)
                             {
                               strcpy(dest,qm) ;
                               return(dest)    ;
                             }

       char    * getqname    (char * const dest)
                             {
                               strcpy(dest,q)  ;
                               return(dest)    ;
                             }

       char    * getflname   (char * const dest)
                             {
                               strcpy(dest,fn) ;
                               return(dest)    ;
                             }


       void     parmset ( int     const argc,                       //Function to set
                          char ** const argv,                       // up parms in the
                          char ** const envp  ) ;                   // class

} ;

//
// The parmset function is the part of the parms class
//     that builds the parameters
//
// Valid parms are :
//
//            /Maaaaa     -> The Queue Manager Name
//            /Qaaaaa     -> The Queue Name
//            /Faaaaa     -> The File  Name
//            /S b - 0 n  -> The Syncpoint Interval (Browse,Outside,end,int)
//            /V + -      -> Message Displaying (Y,N)
//            /Z + -      -> Zero Length record stopping (Y,N)
//
void parms::parmset ( int     const argc,
                      char ** const argv,
                      char ** const envp  )
 {
    int     i = 0   ;
    char  * p = 0   ;
    char    c = ' ' ;
    char    s = ' ' ;
    char    v = ' ' ;

    for ( i=1 ; i < argc ; i++ )     // loop through all the parms
        {
          p = argv??(i??) ;          // start of current parm
          c = *p          ;          // should be the selector, so ignore it
          s = *(++p)      ;          // the parm key
          p++             ;          // start of parm data
          v = *p          ;          // first of the parm data

          switch ( s )
           {
             case    'M'  :      // Queue Manager
             case    'm'  :

                             strncpy(this->qm,p,MQ_Q_MGR_NAME_LENGTH) ;
                             break                                    ;

             case    'Q'  :      // Queue
             case    'q'  :

                             strncpy(this->q,p,MQ_Q_NAME_LENGTH) ;
                             break                               ;

             case    'f'  :      // Queue
             case    'F'  :

                             strncpy(this->fn,p,MAXFILENAME) ;
                             break                           ;


             case    's'  :      // Syncpoint Interval
             case    'S'  :

                             switch ( v )
                                {
                                  case 'b'  :     //Browse
                                  case 'B'  :
                                              syncint = -2         ;
                                              break                ;

                                  case '-'  :     //Outside UOW
                                              syncint = -1         ;
                                              break                ;

                                  default   :     //End of every n
                                              syncint = getnumb(p) ;
                                              break                ;
                                }
                             break ;

             case    'v'  :      // Message usage
             case    'V'  :

                             switch ( v )
                                {

                                  case '+'  :     //Yes
                                              verbose = 1 ;
                                              break       ;

                                  case '-'  :     //No
                                              verbose = 0 ;
                                              break       ;

                                  default   : break       ;
                                }
                             break ;

             case    'z'  :      // Zero Length Record stopping
             case    'Z'  :

                             switch ( v )
                                {

                                  case '+'  :     //Yes
                                              zrl   = 1 ;
                                              break     ;

                                  case '-'  :     //No
                                              zrl  = 0  ;
                                              break     ;

                                  default   : break     ;
                                }
                             break ;


             default :
                        break  ;       // unknown parm, so ignore it
           } ;

        } ;

    return ;

 }

//
// The getnumb private function within the parms class
//     merely extracts the numeric part of a /Pnnnn
//
FULLWORD parms::getnumb( char * const ptr )
 {
    FULLWORD   numb = 0   ;
    char     * p    = 0   ;
    char       c    = ' ' ;

    for ( p = ptr ; *p != '\0' ; p++ )
        {
          c = *p ;
          if ( c == ' ' ) break ;
          if ( ( c >= '0' ) && ( c <= '9' ) )
                numb = ( numb * 10 ) + ( c - '0' ) ;
        }
    return ( numb ) ;
  }



//
// RAHMQO Class Library
//

class rahmqo
 {
   public :

    char        qm  ??(MQ_Q_MGR_NAME_LENGTH+1??) ; //QM name
    char        q   ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name
    MQHCONN     qmh                              ; //QM handle
    MQHOBJ      qh                               ; //Q  handle
    MQLONG      rc                               ; //Return Code
    MQLONG      ac                               ; //Reason Code
    MQLONG      lastrc                           ; //Prev Return Code
    MQLONG      lastac                           ; //Prev Reason Code


    rahmqo ( void )               ;  //Contructor for new  use of QM
    rahmqo ( MQHCONN const uqmh)  ;  //Contructor for same use of QM

    MQLONG      conn ( void )               ;  //Connect    to QM
    MQLONG      conn ( char * const uqm )   ;  //  with QM name supplied

    MQLONG      disc ( void )               ;  //Disconnect from QM

    void        cleanup ( void )            ;  //Close and Disconnect

    MQOD        odopen                      ;  //OD for MQOPEN usage
    MQLONG      open (MQLONG const uopts )  ;  //MQOPEN call
    MQLONG      open (char * const uq    ,     //  with Q name supplied
                      MQLONG const uopts )  ;  //
    MQLONG      open (MQOD * const uod   ,     //  with OD and options
                      MQLONG const uopts )  ;  //

    MQLONG      close (MQLONG const uopts ) ;  //MQCLOSE call

    MQLONG      commit   ( void ) ; //Syncpoint UOW
    MQLONG      rollback ( void ) ; //Backout   UOW

    MQLONG      inq ( MQLONG   const  uselnum ,             //MQINQ call
                      MQLONG          usels   ,
                      MQLONG   const  uinum   ,
                      MQLONG *        uiattr  ,
                      MQLONG   const  ucharl  ,
                      char   *        ucattr  ) ;

    MQLONG      set ( MQLONG   const uselnum  ,             //MQSET call
                      MQLONG         usels    ,
                      MQLONG   const uinum    ,
                      MQLONG *       uiattr   ,
                      MQLONG   const ucharl   ,
                      char   *       ucattr   ) ;

    MQMD        pmd                             ;           //PUT Message Dwescriptor
    MQLONG      put ( MQPMO  * const  uopts   ,             //MQPUT call
                      MQLONG   const  bufferl ,
                      char   * const  buffer  ) ;

    MQLONG      put ( MQMD   *        upmd    ,
                      MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  ) ;

                                                            //PUT Zero-Length record
    MQLONG      putz ( MQPMO  * const  uopts     ) ;        //No Tag, Use Instance MD

    MQLONG      putz ( MQMD   *        upmd      ,          //No Tag, use supplied MD
                       MQPMO  * const  uopts     ) ;

    MQLONG      putz ( MQPMO    * const  uopts   ,          //Tag, Use Instance MD
                       MQBYTE24 * const  utag    ) ;

    MQLONG      putz ( MQMD     *        upmd    ,          //Tag, use supplied MD
                       MQPMO    * const  uopts   ,
                       MQBYTE24 * const  utag    ) ;

    MQMD        bmd                                  ;      //Browse Message Descriptor
    MQGMO       bmo                                  ;      //Browse MQGET options
    MQLONG      browse ( MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  ) ;

    MQLONG      browse ( MQMD   *        ubmd      ,
                         MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  ) ;


    MQMD        gmd                               ;         //GET Message Descriptor
    MQLONG      get ( MQGMO  * const  uopts     ,           //MQGET call
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  ) ;

    MQLONG      get ( MQMD   *        ugmd      ,
                      MQGMO  * const  uopts     ,
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  ) ;


    double      gettimeconn     () { return (time_conn     ) ; }
    double      gettimedisc     () { return (time_disc     ) ; }
    double      gettimeopen     () { return (time_open     ) ; }
    double      gettimeclose    () { return (time_close    ) ; }
    double      gettimeinq      () { return (time_inq      ) ; }
    double      gettimeset      () { return (time_set      ) ; }
    double      gettimeput      () { return (time_put      ) ; }
    double      gettimeget      () { return (time_get      ) ; }
    double      gettimecommit   () { return (time_commit   ) ; }
    double      gettimerollback () { return (time_rollback ) ; }
    double      getcputconn     () { return (cput_conn     / CLOCKS_PER_SEC) ; }
    double      getcputdisc     () { return (cput_disc     / CLOCKS_PER_SEC) ; }
    double      getcputopen     () { return (cput_open     / CLOCKS_PER_SEC) ; }
    double      getcputclose    () { return (cput_close    / CLOCKS_PER_SEC) ; }
    double      getcputinq      () { return (cput_inq      / CLOCKS_PER_SEC) ; }
    double      getcputset      () { return (cput_set      / CLOCKS_PER_SEC) ; }
    double      getcputput      () { return (cput_put      / CLOCKS_PER_SEC) ; }
    double      getcputget      () { return (cput_get      / CLOCKS_PER_SEC) ; }
    double      getcputcommit   () { return (cput_commit   / CLOCKS_PER_SEC) ; }
    double      getcputrollback () { return (cput_rollback / CLOCKS_PER_SEC) ; }

 private :

    double      time_conn      ;
    double      time_disc      ;
    double      time_open      ;
    double      time_close     ;
    double      time_inq       ;
    double      time_set       ;
    double      time_put       ;
    double      time_get       ;
    double      time_commit    ;
    double      time_rollback  ;


    double      cput_conn      ;
    double      cput_disc      ;
    double      cput_open      ;
    double      cput_close     ;
    double      cput_inq       ;
    double      cput_set       ;
    double      cput_put       ;
    double      cput_get       ;
    double      cput_commit    ;
    double      cput_rollback  ;
 } ;

//
// rahmqo member function definitions
//

//
// Constructor functions
//
//     without a parm, then this is a new use of a QM
//     with          , then this is an additional access on a
//                     previously connected QM
//

rahmqo::rahmqo ( void )
{
  memset(qm,0,sizeof(qm)) ;
  memset(q ,0,sizeof(q )) ;
  qmh    = 0              ;
  qh     = 0              ;
  rc     = 0              ;
  ac     = 0              ;
  lastrc = 0              ;
  lastac = 0              ;
  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  memset(&pmd,0,sizeof(pmd))                                   ;
  memset(&bmd,0,sizeof(bmd))                                   ;
  memset(&gmd,0,sizeof(gmd))                                   ;

  cput_conn      = 0 ;
  cput_disc      = 0 ;
  cput_open      = 0 ;
  cput_close     = 0 ;
  cput_inq       = 0 ;
  cput_set       = 0 ;
  cput_put       = 0 ;
  cput_get       = 0 ;
  cput_commit    = 0 ;
  cput_rollback  = 0 ;
  time_conn      = 0 ;
  time_disc      = 0 ;
  time_open      = 0 ;
  time_close     = 0 ;
  time_inq       = 0 ;
  time_set       = 0 ;
  time_put       = 0 ;
  time_get       = 0 ;
  time_commit    = 0 ;
  time_rollback  = 0 ;

  return             ;
}

rahmqo::rahmqo ( MQHCONN const uqmh )
{
  memset(qm,0,sizeof(qm)) ;
  memset(q ,0,sizeof(q )) ;
  qmh    = uqmh           ;
  qh     = 0              ;
  rc     = 0              ;
  ac     = 0              ;
  lastrc = 0              ;
  lastac = 0              ;
  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  memset(&pmd,0,sizeof(pmd))                                   ;
  memset(&bmd,0,sizeof(bmd))                                   ;
  memset(&gmd,0,sizeof(gmd))                                   ;

  cput_conn      = 0 ;
  cput_disc      = 0 ;
  cput_open      = 0 ;
  cput_close     = 0 ;
  cput_inq       = 0 ;
  cput_set       = 0 ;
  cput_put       = 0 ;
  cput_get       = 0 ;
  cput_commit    = 0 ;
  cput_rollback  = 0 ;
  time_conn      = 0 ;
  time_disc      = 0 ;
  time_open      = 0 ;
  time_close     = 0 ;
  time_inq       = 0 ;
  time_set       = 0 ;
  time_put       = 0 ;
  time_get       = 0 ;
  time_commit    = 0 ;
  time_rollback  = 0 ;

  return             ;
}

//
// rahmqo destructor functions
//
//   these are not provided
//
//    (Note that a disc call removes access to ALL
//     instances of RAHMQO sharing the Handle. as I
//     am not aiming at a 'proper' C++ implementation
//     I have not provided code to cope with this event).
//

//
// rahmqo functions to access a Queue Manager
//

//
// conn  will connect to a Queue Manager
//       saving the QMH
//
//       The version with no parms will use the
//           pre-set Queue Manager name
//
//       The alternative access is to quote a
//           Queue Manager Name which is the
//           one to be used
//

MQLONG  rahmqo::conn ( char * const uqm )
{
  double c1, c2, t1, t2         ;
  strncpy(qm,uqm,sizeof(qm)-1)  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCONN ( qm, &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_conn += 1.0000 * ( c2 - c1 )      ;
  time_conn += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::conn ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCONN ( qm, &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_conn += 1.0000 * ( c2 - c1 )      ;
  time_conn += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


//
// disc  will disconnect from a Queue Manager
//       deleting the QMH
//


MQLONG  rahmqo::disc ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQDISC ( &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_disc += 1.0000 * ( c2 - c1 )      ;
  time_disc += 1.0000 * difftime(t2,t1)  ;

  qmh = 0     ;

  return (ac) ;
}

//
// cleanup will Close the Q (if known) and Disconnect from
//         the QM (if known). This will effect other instances
//         which were created with a QM handle
//

void    rahmqo::cleanup ( void )
{
  double c1, c2, t1, t2         ;

  if ( qh  != 0 ) close(MQCO_NONE) ;
  if ( qmh != 0 ) disc ()          ;
  return                           ;
}

//
// commit will Syncpoint ALL accesses to the
//       QM on the given QMH (this will effect
//       other instances of the Class if the
//       QMH is being shared) in the current UOW
//

MQLONG  rahmqo::commit ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCMIT ( qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_commit += 1.0000 * ( c2 - c1 )      ;
  time_commit += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// rollback will backout ALL accesses to the
//       QM on the given QMH (this will effect
//       other instances of the Class if the
//       QMH is being shared) in the current UOW
//

MQLONG  rahmqo::rollback ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQBACK ( qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_rollback += 1.0000 * ( c2 - c1 )      ;
  time_rollback += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// open  will provide access to a Queue
//
//       The OPEN options are always supplied
//           by the caller.
//
//       If the version without a Queue Name is
//          used, then the Name from the Instance
//          is taken.
//

MQLONG  rahmqo::open ( MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  strcpy(odopen.ObjectName,q)                                  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqo::open ( char * const uq    ,
                       MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  strncpy(q,uq,sizeof(q)-1)   ;

  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  strcpy(odopen.ObjectName,q)                                  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqo::open ( MQOD * const uod   ,
                       MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  memcpy(&odopen,uod,sizeof(odopen)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  memcpy(uod,&odopen,sizeof(odopen))     ;
  return (ac) ;
}

//
// close will stop access to a Queue
//
//       The CLOSE options are always supplied
//           by the caller.
//

MQLONG  rahmqo::close ( MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCLOSE ( qmh, &qh, uopts, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_close += 1.0000 * ( c2 - c1 )      ;
  time_close += 1.0000 * difftime(t2,t1)  ;

  qh = 0      ;

  return (ac) ;
}

//
// inq   will inquire upon the Attributes of
//       a Queue etc.
//
//       Note that ALL the Selectors stuff
//       has to be supplied by the caller
//

MQLONG  rahmqo::inq ( MQLONG   const  uselnum ,
                      MQLONG          usels   ,
                      MQLONG   const  uinum   ,
                      MQLONG *        uiattr  ,
                      MQLONG   const  ucharl  ,
                      char   *        ucattr  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQINQ ( qmh, qh, uselnum, &usels ,
                   uinum  , uiattr ,
                   ucharl , ucattr ,
                                     &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_inq += 1.0000 * ( c2 - c1 )      ;
  time_inq += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}
//
// set   will set the Attributes of a Queue etc.
//
//       Note that ALL the Selectors stuff
//       has to be supplied by the caller
//

MQLONG  rahmqo::set ( MQLONG   const   uselnum ,
                      MQLONG           usels   ,
                      MQLONG   const   uinum   ,
                      MQLONG *         uiattr  ,
                      MQLONG   const   ucharl  ,
                      char   *         ucattr  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQSET ( qmh, qh, uselnum , &usels  ,
                   uinum   , uiattr  ,
                   ucharl  , ucattr  ,
                                       &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_set += 1.0000 * ( c2 - c1 )      ;
  time_set += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


//
// put       will write a Message into a Queue
//
//           The Message descriptor may either
//           be supplied or the instance version
//           used.
//

MQLONG  rahmqo::put ( MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, bufferl, buffer, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::put ( MQMD   *        upmd    ,
                      MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, bufferl, buffer, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// putz      will write a Zero-Length Message into a Queue
//
//           If a Tag is supplied, then this is used as the Message Descriptor
//              this must be an area of MQBYTE24 (24 bytes long)
//
//           The Message descriptor may either
//           be supplied or the instance version
//           used.
//

MQLONG  rahmqo::putz ( MQPMO  * const  uopts )               //No Tag, Use Instance MD
{
  double c1, c2, t1, t2         ;

  memset(&this->pmd.MsgId,0,sizeof(this->pmd.MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::putz ( MQMD   *        upmd    ,              //No Tag, use supplied MD
                       MQPMO  * const  uopts   )
{
  double c1, c2, t1, t2         ;

  memset(upmd->MsgId,0,sizeof(upmd->MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqo::putz ( MQPMO    * const  uopts   ,            //Tag, Use Instance MD
                       MQBYTE24 * const  utag    )
{
  double c1, c2, t1, t2         ;

  memcpy(&this->pmd.MsgId,utag,sizeof(this->pmd.MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::putz ( MQMD     *        upmd    ,            //Tag, use supplied MD
                       MQPMO    * const  uopts   ,
                       MQBYTE24 * const  utag    )
{
  double c1, c2, t1, t2         ;

  memcpy(upmd->MsgId,utag,sizeof(upmd->MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
//    Browse    will non-destructively read a Queue
//
//              Versions are supplied that use an
//              instance Message dewscriptor, or
//              one supplied by the caller
//

MQLONG  rahmqo::browse ( MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&bmo,0,sizeof(bmo))                                 ;
  memcpy(&bmo.StrucId,MQGMO_STRUC_ID,sizeof(bmo.StrucId))    ;
  bmo.Version        = MQGMO_VERSION_1                       ;
  bmo.Options        = MQGMO_NO_WAIT+MQGMO_BROWSE_NEXT+MQGMO_ACCEPT_TRUNCATED_MSG+
                       MQGMO_FAIL_IF_QUIESCING               ;
  bmo.WaitInterval   = MQWI_UNLIMITED                        ;

  memset(&bmd,0,sizeof(bmd))                                 ;
  memcpy(&bmd.StrucId,MQMD_STRUC_ID,sizeof(bmd.StrucId))     ;
  bmd.Version        = MQMD_VERSION_1                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, &bmd, &bmo, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::browse ( MQMD   *        ubmd      ,
                         MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&bmo,0,sizeof(bmo))                                 ;
  memcpy(&bmo.StrucId,MQGMO_STRUC_ID,sizeof(bmo.StrucId))    ;
  bmo.Version        = MQGMO_VERSION_1                       ;
  bmo.Options        = MQGMO_NO_WAIT+MQGMO_BROWSE_NEXT+MQGMO_ACCEPT_TRUNCATED_MSG+
                       MQGMO_CONVERT+MQGMO_FAIL_IF_QUIESCING ;
  bmo.WaitInterval   = MQWI_UNLIMITED                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, ubmd, &bmo, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::get ( MQGMO  * const  uopts     ,           //MQGET call
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&gmd,0,sizeof(gmd))                                 ;
  memcpy(&gmd.StrucId,MQMD_STRUC_ID,sizeof(gmd.StrucId))     ;
  bmo.Version        = MQMD_VERSION_1                        ;

  memset(&gmd,0,sizeof(gmd))                                 ;
  memcpy(&gmd.StrucId,MQMD_STRUC_ID,sizeof(gmd.StrucId))     ;
  gmd.Version        = MQMD_VERSION_1                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, &gmd, uopts, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqo::get ( MQMD   *        ugmd      ,
                      MQGMO  * const  uopts     ,
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, ugmd, uopts, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// End of RAHMQO Class definition and functions
//



//
// Main procedure - Append a Queue to a file
//
int  main ( int     const argc,
            char ** const argv,
            char ** const envp  )
{
 parms       myparms        ;
 rahmqo      mymq           ;

 char        qm  ??(MQ_Q_MGR_NAME_LENGTH+1??) ; //QM name
 char        q   ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name
 char        fn  ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name

 FULLWORD    syncint = 0    ;
 FULLWORD    persist = 0    ;
 FULLWORD    zrl     = 1    ;
 MQLONG      opts    = 0    ;
 MQGMO       gmo            ;

 MQLONG      maxmsgl = 0    ;
 char     *  buffer  = 0    ;
 FILE     *  fhandle = 0    ;
 FULLWORD    maxnum  = 0    ;

 double      totcput = 0    ;
 double      tottime = 0    ;
 double      totc1   = 0    ;
 double      totc2   = 0    ;
 double      tote1   = 0    ;
 double      tote2   = 0    ;

 MQLONG      mqrc    = 0    ;
 MQLONG      mqac    = 0    ;
 MQLONG      ac      = 0    ;
 MQLONG      rc      = 0    ;

 FULLWORD    syncno  = 0    ;
 FULLWORD    totsync = 0    ;
 FULLWORD    recno   = 0    ;
 FULLWORD    zrlno   = 0    ;

 totc1 = clock()    ;                      //Set Timers
 tote1 = time(NULL) ;

 myparms.parmset(argc, argv, envp)      ;  //set the parms

 myparms.getqmname(qm)          ;    //Get the data
 myparms.getqname (q )          ;    // from the
 myparms.getflname(fn)          ;    // parms
 syncint = myparms.getsyncint() ;    //
 zrl     = myparms.getzrl()     ;    //

 if ( myparms.isverbose() )          //Display program parms
    {
      cout << "Program parms are: " ;
      for ( int i=0 ; i<argc ; cout << argv??(i++??) << " " ) ;
      cout << endl ;
    }

 if ( argc == 1 )                    //Show parms ONLY if none given
    {
      cout << "\nMQOQTOFL will append a Queue to a file \n\n"
           << "Parms for MQOQTOFL are /M<queue manager>  /Q<queue> \n"
           << "                       /F<file name>      /S<sync interval> \n"
           << "                       /V<verbose>        /Z<ZRL stopping> \n\n"
           << " /SB Browse Q    /S- Run outside UOW     /S0 Sync at end     /Sn Sync every n \n"
           << " /V+ display     /V- no messages         /Z+ Stop on 1st ZRL /Z- Ignore ZRLs \n\n"
           ;
      return (0) ;
    }


 switch ( syncint )              //Set Get options
  {
    case -2 :
               break                              ;

    case -1 :
               opts  = MQGMO_NO_SYNCPOINT         +
                       MQGMO_NO_WAIT              +
                       MQGMO_ACCEPT_TRUNCATED_MSG +
                       MQGMO_FAIL_IF_QUIESCING    ;
               break                              ;

    default :
               opts  = MQGMO_SYNCPOINT            +
                       MQGMO_NO_WAIT              +
                       MQGMO_ACCEPT_TRUNCATED_MSG +
                       MQGMO_FAIL_IF_QUIESCING    ;
               break                              ;
  }

  if ( myparms.isverbose() ) switch ( syncint )        //Display UoW info
   {
     case -2 :
              cout << "Messages to be read by a Browse" << endl   ;
              break                                               ;

     case -1 :
              cout << "Messages to be read Outside of a UoW" << endl  ;
              break                                                   ;

     case 0  :
              cout << "Messages(within a UoW) are Syncpointed after all have been read from the Q" << endl  ;
              break                                                                                         ;

     default :
              cout << "Messages(within a UoW) are Syncpointed every " << syncint << " messages" << endl ;
              break                                                                                     ;
   }

  if ( myparms.isverbose() ) switch ( zrl )        //Display Zero Length Record Stopping
   {
     case 0  :
              cout << "Zero Length records are ignored" << endl  ;
              break                                                                                         ;

     default :
              cout << "Operations will stop if a Zero Length Record is encountered " << endl ;
              break                                                                                     ;
   }

 if ( rc == 0 )                      // Connect to QM
    {
      if ( myparms.isverbose() ) cout << "Connecting to QM " << qm << " " ;
      ac   = mymq.conn(qm)   ;
      mqrc = mymq.lastrc     ;
      mqac = mymq.lastrc     ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Open the Q for INQ/Get access
    {
      switch ( syncint )
       {
         case  -2  :
                    if ( myparms.isverbose() ) cout << "Opening Q " << q << " for Non-Destructive access "   ;
                    ac   = mymq.open(q,(MQOO_INQUIRE+MQOO_INPUT_SHARED
                                                    +MQOO_BROWSE+MQOO_SAVE_ALL_CONTEXT
                                                    +MQOO_FAIL_IF_QUIESCING))                                ;
                    mqrc = mymq.lastrc     ;
                    mqac = mymq.lastrc     ;
                    break                  ;

         default   :
                    if ( myparms.isverbose() ) cout << "Opening Q " << q << " for Destructive access " ;
                    ac   = mymq.open(q,(MQOO_INQUIRE+MQOO_INPUT_EXCLUSIVE
                                                    +MQOO_BROWSE+MQOO_SAVE_ALL_CONTEXT
                                                    +MQOO_FAIL_IF_QUIESCING))                          ;
                    mqrc = mymq.lastrc     ;
                    mqac = mymq.lastrc     ;
                    break                  ;
       }

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Get MaxMessage Size
    {
      MQLONG   tselcount        = 1                   ;
      MQLONG   tsels            = MQIA_MAX_MSG_LENGTH ;
      MQLONG   tintcount        = 1                   ;
      MQLONG   tintattr         = 0                   ;
      MQLONG   tcharlen         = 0                   ;
      char     tcharattr        = '\0'                ;

      if ( myparms.isverbose() ) cout << "Obtaining Q Max Message Length " ;
      ac   =  mymq.inq(tselcount, tsels,
                                         tintcount, &tintattr  ,
                                         tcharlen , &tcharattr ) ;

      mqrc = mymq.lastrc          ;
      mqac = mymq.lastrc          ;

      maxmsgl = tintattr          ;

      if ( ac != 0 ) rc = ac      ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << " Max Len is " << maxmsgl << endl ;

      if ( maxmsgl > MAXUHALFWORD )
         {
           maxmsgl = MAXUHALFWORD ;
           if ( myparms.isverbose() ) cout << "which is forced to max supported size of " << MAXUHALFWORD << endl ;
         }
    }

 if ( rc == 0 )                      // Get Number of Messages in the Q
    {
      MQLONG   tselcount        = 1                    ;
      MQLONG   tsels            = MQIA_CURRENT_Q_DEPTH ;
      MQLONG   tintcount        = 1                    ;
      MQLONG   tintattr         = 0                    ;
      MQLONG   tcharlen         = 0                    ;
      char     tcharattr        = '\0'                 ;

      if ( myparms.isverbose() ) cout << "Obtaining current Q depth " ;
      ac   =  mymq.inq(tselcount, tsels,
                                         tintcount, &tintattr  ,
                                         tcharlen , &tcharattr ) ;

      mqrc = mymq.lastrc          ;
      mqac = mymq.lastrc          ;

      maxnum  = tintattr          ;

      if ( ac != 0 ) rc = ac      ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << " Depth is " << maxnum << endl ;

    }

 if ( rc == 0 )                      // Getmain the Buffer (always max file size)
    {
      if ( myparms.isverbose() ) cout << "Getmaining the File/Q Buffer " ;
      buffer = (char *)malloc(MAXUHALFWORD) ;
      if ( buffer == 0 ) rc = -8    ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Open the Output file
    {
      if ( myparms.isverbose() ) cout << "Opening file " << fn << " " ;
      fhandle = fopen(fn,"ab")        ;
      if ( fhandle == NULL ) rc = -9  ;
      if ( myparms.isverbose() ) cout << ((rc != 0) ? "Operation Failed"
              : "File now Open for Output. Q Read+File Write starting" ) << endl ;
    }


 if ( rc == 0 )                      // Read Q and Write File
    {
      for ( recno = 0 , syncno = 0 ; recno < maxnum ; recno++, syncno++ )
          {
            MQLONG   i      = 0   ;
            char     exiter = 'N' ;
            char   * c      = 0   ;
            char   * p      = 0   ;
            MQLONG   mrc    = 0   ;
            MQLONG   frc    = 0   ;
            FULLWORD reclen = 0   ;

            switch ( syncint )
             {
               case    -2   :             //Browse
                              memset(buffer,0,MAXUHALFWORD)                       ;
                              reclen   = 0                                        ;
                              mrc = mymq.browse ( MAXUHALFWORD, buffer, &reclen ) ;
                              break                                               ;

               case    -1   :             //Get outside of Syncpoint
                              memset(&gmo,0,sizeof(gmo))                                        ;
                              memcpy(&gmo.StrucId,MQGMO_STRUC_ID,sizeof(gmo.StrucId))           ;
                              gmo.Version        = MQGMO_VERSION_1                              ;
                              gmo.Options        = opts                                         ;
                              gmo.WaitInterval   = MQWI_UNLIMITED                               ;

                              memset(&mymq.gmd,0,sizeof(mymq.gmd))                              ;
                              memcpy(&mymq.gmd.StrucId,MQMD_STRUC_ID,sizeof(mymq.gmd.StrucId))  ;
                              mymq.gmd.Version   = MQMD_VERSION_1                               ;

                              memset(buffer,0,MAXUHALFWORD)                                     ;
                              reclen   = 0                                                      ;
                              mrc = mymq.get ( &gmo, MAXUHALFWORD, buffer, &reclen )            ;
                              break                                                             ;

               default      :             //Get inside of Syncpoint
                              memset(&gmo,0,sizeof(gmo))                                        ;
                              memcpy(&gmo.StrucId,MQGMO_STRUC_ID,sizeof(gmo.StrucId))           ;
                              gmo.Version        = MQGMO_VERSION_1                              ;
                              gmo.Options        = opts                                         ;
                              gmo.WaitInterval   = MQWI_UNLIMITED                               ;

                              memset(&mymq.gmd,0,sizeof(mymq.gmd))                              ;
                              memcpy(&mymq.gmd.StrucId,MQMD_STRUC_ID,sizeof(mymq.gmd.StrucId))  ;
                              mymq.gmd.Version   = MQMD_VERSION_1                               ;

                              memset(buffer,0,MAXUHALFWORD)                                     ;
                              reclen   = 0                                                      ;
                              mrc = mymq.get ( &gmo, MAXUHALFWORD, buffer, &reclen )            ;
                              break                                                             ;

             }

            switch ( mrc )
             {
               case MQRC_NONE                   :
                    rc   = 0      ;
                    break         ;

               case MQRC_FORMAT_ERROR           :
                    rc   = 0      ;
                    break         ;

               case MQRC_TRUNCATED_MSG_ACCEPTED :
                    if ( myparms.isverbose() ) cout << "Message " << recno+1 << " of length " << reclen
                                                    << " too big  for file maximum "
                                                    << "length of " << MAXUHALFWORD << endl ;
                    exiter = 'F'  ;
                    mrc = - 10    ;
                    rc  = mrc     ;
                    break         ;

               default                          :
                    if ( myparms.isverbose() ) cout << "MQ error RC=" << mrc << " on message " << recno+1 << endl ;
                    exiter = 'F'  ;
                    rc  = mrc     ;
                    break         ;
             }

            if (    ( rc     == 0 )
                 && ( reclen == 0 ) ) zrlno ++ ;     //Up Zero Length record count

            if (    ( rc     == 0 )
                 && ( reclen == 0 )
                 && ( zrl    != 0 ) ) exiter = 'Y' ;     //Zero Length record -> exit if so requested


            if (    ( rc     ==  0  )
                 && ( exiter != 'Y' )
                 && ( recno  !=  0  )
                 && ( reclen !=  0  ) ) //no \n before 1st real record!
               {                                  // done this way to avoid 2 \n s
                  frc = fputs("\n",fhandle) ;     // at EOF
                  if ( frc == EOF )
                     {
                       if ( myparms.isverbose() ) cout << "File Write failed on \\n " << recno+1 << endl ;
                       rc = - 11 ;
                     }
               }

            if ( ( rc == 0 ) && ( exiter != 'Y' ) && ( reclen != 0 ) )
            {
                 frc = fwrite(buffer,1,reclen,fhandle) ;
                 if ( frc != reclen )
                    {
                       if ( myparms.isverbose() ) cout << "File Write failed on message " << recno+1 << endl ;
                       rc = - 12 ;
                    }
                }

                                                //Syncpoint if its
                                                //to be done after
                                                //each GET, or if
                                                //the required number
                                                //of GETS have been
                                                //done (or EOQ hit)

            if ( ( rc == 0 ) && ( syncint >= 0 ) )
               {
                 if (    ( ( syncint >= 1       ) && ( syncno  >= syncint ) )
                      || (   exiter  == 'Y'     )  )
                    {
                      syncno = 0         ;
                      totsync++          ;
                      rc = mymq.commit() ;
                      if ( rc != 0 )
                         {
                           if ( myparms.isverbose() ) cout << "Syncpoint Error RC=" << rc << endl ;
                         }
                    }
               } // End of Syncpoint Block

                                                //If there was a
                                                //failure of any sort
                                                //whilsr running
                                                //inside a Uow, then
                                                //Backout any updates

              if ( ( rc != 0 ) && ( syncint >= 0 ) )
                 {
                   if ( myparms.isverbose() ) cout << "Initiating Rollback due to RC " << rc ;
                   exiter = 'X'           ;
                   ac = mymq.rollback() ;
                   if ( myparms.isverbose() ) cout << " Rollback RC=" << ac << endl ;
                 } // End of Rollback Block

              if ( ( rc == 0 ) && ( recno >= maxnum ) ) exiter = 'Y' ; //Exit due to all messages read OK

              if ( exiter != 'N' ) break ; //Get next message
                                           //unless error/EOQ occured

          } // End of Message Processing FOR loop

    } // End of Queue/File Processing Block

if ( ( rc == 0 ) && ( myparms.isverbose() ) ) cout << "All Q records put into file" << endl ;

                                                //End of processing

 if ( rc == 0 )                      // Close the Q
    {
      if ( myparms.isverbose() ) cout << "Closing the Q " ;
      ac   = mymq.close(MQCO_NONE)  ;
      mqrc = mymq.lastrc            ;
      mqac = mymq.lastrc            ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Disconnect from QM
    {
      if ( myparms.isverbose() ) cout << "Disconnecting from QM " ;
      ac   = mymq.disc()            ;
      mqrc = mymq.lastrc            ;
      mqac = mymq.lastrc            ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Close the File
    {
      if ( myparms.isverbose() ) cout << "Closing the file " ;
      ac  = fclose(fhandle)        ;

      if ( ac != 0 ) rc = -13 ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( ( rc == 0 ) && ( myparms.isverbose() ) )  //Issue Good completion message
    {

      totc2 = clock()    ;
      tote2 = time(NULL) ;
      totcput = 1.0000 * ( totc2 - totc1  ) / CLOCKS_PER_SEC   ;
      tottime = 1.0000 * difftime(tote2,tote1)                 ;

      if ( zrlno == 1 ) recno++ ; //As the FOR loop does not go round again if
                                  //one stops on the first ZRL, then the count
                                  //is one to small - so fiddle it here!!!

      cout << recno << " records obtained from Q " << q
           << " (" << zrlno << " Zero Length), so "
           << (recno-zrlno) << " written to file " << fn << "\n" ;
      cout << "CPU time (ms) and Elapsed Time (s) per MQ verb follow:\n"                   ;
      cout << " >Connect    " << mymq.getcputconn     () << " " << mymq.gettimeconn     () << "\n" ;
      cout << " >Disconnect " << mymq.getcputdisc     () << " " << mymq.gettimedisc     () << "\n" ;
      cout << " >Open       " << mymq.getcputopen     () << " " << mymq.gettimeopen     () << "\n" ;
      cout << " >Close      " << mymq.getcputclose    () << " " << mymq.gettimeclose    () << "\n" ;
      if ( totsync == 0 )
           cout << " >Commit     " << mymq.getcputcommit   () << " " << mymq.gettimecommit   () << "\n" ;
           else
           cout << " >Commit     " << mymq.getcputcommit   () << " " << mymq.gettimecommit   () <<
                   " per call "    << mymq.getcputcommit   () / totsync <<
                   " "             << mymq.gettimecommit   () / totsync << "\n" ;
      cout << " >Rollback   " << mymq.getcputrollback () << " " << mymq.gettimerollback () << "\n" ;
      cout << " >Inquire    " << mymq.getcputinq      () << " " << mymq.gettimeinq      () << "\n" ;
      cout << " >Set        " << mymq.getcputset      () << " " << mymq.gettimeset      () << "\n" ;
      cout << " >Get        " <<    mymq.getcputget()               << " " << mymq.gettimeget() <<
              "  per record " <<  ( mymq.getcputget() / (recno+1) ) <<
                          " " <<  ( mymq.gettimeget() / (recno+1) ) << "\n" ;
      cout << " >Put        " <<    mymq.getcputput()               << " " << mymq.gettimeput() <<
              "  per record " <<  ( mymq.getcputput() / (recno+1) ) <<
                          " " <<  ( mymq.gettimeput() / (recno+1) ) << "\n" ;
      cout << "Total MQ CPU time " << (   mymq.getcputconn     ()
                                        + mymq.getcputdisc     ()
                                        + mymq.getcputopen     ()
                                        + mymq.getcputclose    ()
                                        + mymq.getcputcommit   ()
                                        + mymq.getcputrollback ()
                                        + mymq.getcputinq      ()
                                        + mymq.getcputset      ()
                                        + mymq.getcputget      ()
                                        + mymq.getcputput      () ) << "s "
           << "Total MQ Elapsed time " << (   mymq.gettimeconn     ()
                                            + mymq.gettimedisc     ()
                                            + mymq.gettimeopen     ()
                                            + mymq.gettimeclose    ()
                                            + mymq.gettimecommit   ()
                                            + mymq.gettimerollback ()
                                            + mymq.gettimeinq      ()
                                            + mymq.gettimeset      ()
                                            + mymq.gettimeget      ()
                                            + mymq.gettimeput      () )  << "s\n" ;
      cout << "Total CPU " << totcput << "s, Total elapsed time " << tottime << "s" << endl ;

    }

 fclose(fhandle)                       ; //Close the file
 if ( buffer != 0 ) free(buffer)       ; //Freemain the buffer
 mymq.cleanup()                        ; //Stop all accesses

 return(rc) ;                           //Return RC

}

// End of MQOQTOFL.CPP source
